@@ -2,13 +2,18 @@ import os
 import re
 import ast
 import glob
-import builtins
+
+from xonsh.built_ins import XSH
+from xonsh.parsers.completion_context import CommandContext
 
 import xonsh.tools as xt
 import xonsh.platform as xp
 import xonsh.lazyasd as xl
 
-from xonsh.completers.tools import get_filter_function
+from xonsh.completers.tools import (
+    RichCompletion,
+    contextual_completer,
+)
 
 
 @xl.lazyobject
@@ -22,7 +27,7 @@ def PATTERN_NEED_QUOTES():
 
 def cd_in_command(line):
     """Returns True if "cd" is a token in the line, False otherwise."""
-    lexer = builtins.__xonsh__.execer.parser.lexer
+    lexer = XSH.execer.parser.lexer
     lexer.reset()
     lexer.input(line)
     have_cd = False
@@ -76,7 +81,7 @@ def _path_from_partial_string(inp, pos=None):
     except (SyntaxError, ValueError):
         return None
     if isinstance(val, bytes):
-        env = builtins.__xonsh__.env
+        env = XSH.env
         val = val.decode(
             encoding=env.get("XONSH_ENCODING"), errors=env.get("XONSH_ENCODING_ERRORS")
         )
@@ -98,7 +103,7 @@ def _normpath(p):
         p = os.path.join(os.curdir, p)
     if trailing_slash:
         p = os.path.join(p, "")
-    if xp.ON_WINDOWS and builtins.__xonsh__.env.get("FORCE_POSIX_PATHS"):
+    if xp.ON_WINDOWS and XSH.env.get("FORCE_POSIX_PATHS"):
         p = p.replace(os.sep, os.altsep)
     return p
 
@@ -113,20 +118,17 @@ def _startswithnorm(x, start, startlow=None):
     return x.startswith(start)
 
 
-def _env(prefix):
-    if prefix.startswith("$"):
-        key = prefix[1:]
-        return {
-            "$" + k for k in builtins.__xonsh__.env if get_filter_function()(k, key)
-        }
-    return ()
-
-
 def _dots(prefix):
+    complete_dots = XSH.env.get("COMPLETE_DOTS", "matching").lower()
+    if complete_dots == "never":
+        return ()
     slash = xt.get_sep()
     if slash == "\\":
         slash = ""
-    if prefix in {"", "."}:
+    prefixes = {"."}
+    if complete_dots == "always":
+        prefixes.add("")
+    if prefix in prefixes:
         return ("." + slash, ".." + slash)
     elif prefix == "..":
         return (".." + slash,)
@@ -136,7 +138,7 @@ def _dots(prefix):
 
 def _add_cdpaths(paths, prefix):
     """Completes current prefix using CDPATH"""
-    env = builtins.__xonsh__.env
+    env = XSH.env
     csc = env.get("CASE_SENSITIVE_COMPLETIONS")
     glob_sorted = env.get("GLOB_SORTED")
     for cdp in env.get("CDPATH"):
@@ -158,7 +160,7 @@ def _quote_to_use(x):
 
 
 def _is_directory_in_cdpath(path):
-    env = builtins.__xonsh__.env
+    env = XSH.env
     for cdp in env.get("CDPATH"):
         if os.path.isdir(os.path.join(cdp, path)):
             return True
@@ -166,7 +168,7 @@ def _is_directory_in_cdpath(path):
 
 
 def _quote_paths(paths, start, end, append_end=True, cdpath=False):
-    expand_path = builtins.__xonsh__.expand_path
+    expand_path = XSH.expand_path
     out = set()
     space = " "
     backslash = "\\"
@@ -198,8 +200,6 @@ def _quote_paths(paths, start, end, append_end=True, cdpath=False):
         if end != "":
             if "r" not in start.lower():
                 s = s.replace(backslash, double_backslash)
-            if s.endswith(backslash) and not s.endswith(double_backslash):
-                s += backslash
         if end in s:
             s = s.replace(end, "".join("\\%s" % i for i in end))
         s = start + s + end if append_end else start + s
@@ -275,7 +275,7 @@ def _subsequence_match_iter(ref, typed):
 
 def _expand_one(sofar, nextone, csc):
     out = set()
-    glob_sorted = builtins.__xonsh__.env.get("GLOB_SORTED")
+    glob_sorted = XSH.env.get("GLOB_SORTED")
     for i in sofar:
         _glob = os.path.join(_joinpath(i), "*") if i is not None else "*"
         for j in xt.iglobpath(_glob, sort_result=glob_sorted):
@@ -285,7 +285,7 @@ def _expand_one(sofar, nextone, csc):
     return out
 
 
-def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
+def _complete_path_raw(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
     """Completes based on a path name."""
     # string stuff for automatic quoting
     path_str_start = ""
@@ -306,7 +306,7 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
             append_end = False
     tilde = "~"
     paths = set()
-    env = builtins.__xonsh__.env
+    env = XSH.env
     csc = env.get("CASE_SENSITIVE_COMPLETIONS")
     glob_sorted = env.get("GLOB_SORTED")
     prefix = glob.escape(prefix)
@@ -352,9 +352,42 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
         {_normpath(s) for s in paths}, path_str_start, path_str_end, append_end, cdpath
     )
     paths.update(filter(filtfunc, _dots(prefix)))
-    paths.update(filter(filtfunc, _env(prefix)))
     return paths, lprefix
 
 
-def complete_dir(prefix, line, start, end, ctx, cdpath=False):
-    return complete_path(prefix, line, start, end, cdpath, filtfunc=os.path.isdir)
+@contextual_completer
+def complete_path(context):
+    if context.command:
+        return contextual_complete_path(context.command)
+    elif context.python:
+        line = context.python.prefix
+        # simple prefix _complete_path_raw will handle gracefully:
+        prefix = line.rsplit(" ", 1)[1]
+        return _complete_path_raw(prefix, line, len(line) - len(prefix), len(line), {})
+    return set(), 0
+
+
+def contextual_complete_path(command: CommandContext, cdpath=True, filtfunc=None):
+    # ``_complete_path_raw`` may add opening quotes:
+    prefix = command.raw_prefix
+
+    completions, lprefix = _complete_path_raw(
+        prefix,
+        prefix,
+        0,
+        len(prefix),
+        ctx={},
+        cdpath=cdpath,
+        filtfunc=filtfunc,
+    )
+
+    # ``_complete_path_raw`` may have added closing quotes:
+    rich_completions = {
+        RichCompletion(comp, append_closing_quote=False) for comp in completions
+    }
+
+    return rich_completions, lprefix
+
+
+def complete_dir(command: CommandContext):
+    return contextual_complete_path(command, filtfunc=os.path.isdir)
